/*
  GO-WARP: a Time Warp simulator written in Go
  http://pads.cs.unibo.it

  This file is part of GO-WARP.  GO-WARP is free software: you can
  redistribute it and/or modify it under the terms of the GNU General Public
  License as published by the Free Software Foundation, version 2.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

  Copyright 2012, Pietro Ansaloni, Gabriele D'Angelo, Moreno Marzolla
  Computer Science Department, University of Bologna, Italy
*/

package Communication

import(
    "./DT"
    "fmt"
)

const MAXBUFFER = 10000
const MAXALLOCN = 1
var(
    Chanptr *[]chan DT.Message
    lock chan int = make(chan int)
    allocations int = 0
)


func AllocateChans(nChan int) {

    if allocations >= MAXALLOCN { return }

    ch := make ([]chan DT.Message, nChan)	// this is to make the array

    for i:=0; i<nChan; i++ {
        ch[i]=make(chan DT.Message, MAXBUFFER)	// this is to make the chans
    }
    Chanptr = &ch
    allocations++
}


/* Send a message to destination */
func Send(msg *DT.Message) {
    (*Chanptr)[msg.Receiver] <- *msg
}


func Receive(recvid DT.Pid) *DT.Message {
    var msg DT.Message
    var ret *DT.Message
    var ok bool = false

    msg,ok = <- (*Chanptr)[recvid]
    if ok {
        ret = &msg
    } else {
        ret = nil
    }

    return ret
}

/* blocking receive */
func BlockingReceive(recvid DT.Pid) *DT.Message {
    select {
        case msg := <- (*Chanptr)[recvid]:
            return &msg
    }
    fmt.Println("GO-WARP: receive error!")
    return nil
}


func Sync() {
    <- lock
}

func Unlock() {
    lock <- 0
}
